@@ -21,7 +21,6 @@
 # SOFTWARE.
 #
 import collections
-import errno
 import os
 import re
 import signal
@@ -50,6 +49,8 @@ _FILE_MAP = {"IN": "input",
 
 
 class CmdError(RuntimeError):
+    """Exception raised for AtomicCmd specific errors."""
+
     def __init__(self, msg):
         RuntimeError.__init__(self, msg)
 
@@ -144,6 +145,8 @@ class AtomicCmd(object):
         self._running = False
         self._command = map(str, safe_coerce_to_tuple(command))
         self._set_cwd = set_cwd
+        self._terminated = False
+
         if not self._command or not self._command[0]:
             raise ValueError("Empty command in AtomicCmd constructor")
 
@@ -237,6 +240,7 @@ class AtomicCmd(object):
         if self._proc and self._proc.poll() is None:
             try:
                 os.killpg(self._proc.pid, signal.SIGTERM)
+                self._terminated = True
             except OSError:
                 pass  # Already dead / finished process
 
@@ -279,7 +283,7 @@ class AtomicCmd(object):
                         committed_files.add(self._files[key])
                     elif key.startswith("TEMP_OUT_"):
                         fileutils.try_remove(filename)
-        except:
+        except StandardError:
             # Cleanup after failed commit
             for fpath in committed_files:
                 fileutils.try_remove(fpath)
