@@ -23,6 +23,7 @@
 library(ape)
 library(ggplot2)
 library(grid)
+library(methods)
 
 
 TTBar <- setRefClass("TTBar",
@@ -490,7 +491,7 @@ plot.tree <- function(filename, sample_names, padding=0.3)
     pp <- tinytree.default.sizes(pp, tt, "node"=3, "leaf"=4, "line"=0.75)
 
     defaults <- c("line"="black",
-                  "node"="darkgrey",
+                  "node"="grey40",
                   "leaf"="black",
                   "bar"="blue")
     defaults <- tinytree.defaults.collect(tt, defaults, list())
@@ -525,7 +526,7 @@ plot.tree(input_file, sample_names)
 dev.off()
 
 # bitmap is preferred, since it works in a headless environment
-bitmap(paste(output_prefix, ".png", sep=""), height=5, width=5, res=96)
+bitmap(paste(output_prefix, ".png", sep=""), height=6, width=6, res=96, taa=4, gaa=4)
 plot.tree(input_file, sample_names)
 dev.off()
 
