@@ -19,13 +19,17 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+import os
 import tarfile
 
 import pysam
 
 import paleomix.yaml
 
-from paleomix.common.formats.fasta import FASTA
+from paleomix.common.fileutils import \
+    swap_ext
+from paleomix.common.formats.fasta import \
+    FASTA
 from paleomix.tools.zonkey.common import \
     get_sample_names, \
     contig_name_to_plink_name
@@ -83,16 +87,32 @@ class ZonkeyDB(object):
     def __init__(self, filename):
         self.filename = filename
 
+        if not os.path.exists(filename):
+            raise ZonkeyDBError('Database file does not exist')
+        elif not tarfile.is_tarfile(filename):
+            raise ZonkeyDBError('Database file is not a valid tar-file')
+
+        print_info('Reading Zonkey database from %r ...' % (filename,))
+
+        # Warn if file is gzip / bzip2 compressed; gives worse throughput
+        _check_file_compression(filename)
+
         with tarfile.open(filename) as tar_handle:
+            print_info('  - Reading settings ...')
             self.settings = self._read_settings(tar_handle, "settings.yaml")
+            print_info('  - Reading list of contigs ...')
             self.contigs = self._read_contigs_table(tar_handle, "contigs.txt")
+            print_info('  - Reading list of samples ...')
             self.samples = self._read_samples_table(tar_handle, "samples.txt")
-            self.sample_order = self._read_sample_order(tar_handle,
-                                                        "genotypes.txt")
+            print_info('  - Reading mitochondrial sequences ...')
             self.mitochondria = self._read_mitochondria(tar_handle,
                                                         "mitochondria.fasta")
+            print_info('  - Reading emperical admixture distribution ...')
             self.simulations = self._read_simulations(tar_handle,
                                                       "simulations.txt")
+            print_info('  - Determining sample order ...')
+            self.sample_order = self._read_sample_order(tar_handle,
+                                                        "genotypes.txt")
 
         self._cross_validate()
 
@@ -144,6 +164,9 @@ class ZonkeyDB(object):
                                 "both tables: %s"
                                 % (",".join(differences),))
 
+        if self.mitochondria is None:
+            return
+
         for name, record in self.mitochondria.iteritems():
             if name not in self.samples:
                 # Ignore extra reference sequences
@@ -314,9 +337,12 @@ class ZonkeyDB(object):
         return result
 
     def _read_simulations(self, tar_handle, filename):
-        self._check_required_file(tar_handle, filename)
+        try:
+            handle = tar_handle.extractfile(filename)
+        except KeyError:
+            # Missing simulations file is allowed
+            return None
 
-        handle = tar_handle.extractfile(filename)
         header = handle.readline().rstrip().split('\t')
 
         required_keys = set(('NReads', 'K', 'Sample1', 'Sample2', 'HasTS',
@@ -431,6 +457,10 @@ class ZonkeyDB(object):
 
 
 def _validate_mito_bam(data, handle, info):
+    if data.mitochondria is None:
+        # No mitochondrial data .. skip phylogeny
+        return True
+
     references = handle.references
     min_length = min((len(record.sequence))
                      for record in data.mitochondria.itervalues())
@@ -449,14 +479,25 @@ def _validate_mito_bam(data, handle, info):
                       % (bam_contig, bam_length, db_length))
             return False
 
-        for line in pysam.idxstats(handle.filename):
-            name, _, hits, _ = line.strip().split('\t')
+        if not os.path.exists(handle.filename + '.bai') \
+                and not os.path.exists(swap_ext(handle.filename, '.bai')):
+            print_info('    - Attempting to index BAM file %r!'
+                       % (handle.filename,))
+            pysam.index(handle.filename)
+
+        # Workaround for pysam < 0.9 returning list, >= 0.9 returning str
+        for line in "".join(pysam.idxstats(handle.filename)).split('\n'):
+            line = line.strip()
+            if not line:
+                continue
+
+            name, _, hits, _ = line.split('\t')
             if (name == bam_contig) and not int(hits):
-                print_err("ERROR: Mitochondrial BAM (%r) does not contain "
+                print_err("WARNING: Mitochondrial BAM (%r) does not contain "
                           "any reads aligned to contig %r; inferring an "
                           "phylogeny is not possible."
                           % (handle.filename, name))
-                return False
+                return True
 
         info.mt_contig = bam_contig
         info.mt_length = bam_length
@@ -499,3 +540,23 @@ def _validate_nuclear_bam(data, handle, info):
             info.nuclear = True
 
     return True
+
+
+def _check_file_compression(filename):
+    try:
+        with open(filename) as handle:
+            header = handle.read(2)
+
+            if header == "\x1f\x8b":
+                print_warn('\nWARNING:\n'
+                           'Zonkey database file %r is gzip compressed;\n'
+                           'uncompressing the archive is recommended:\n'
+                           '  $ gunzip "%s"\n' % (filename, filename))
+            elif header == "BZ":
+                print_warn('\nWARNING:\n'
+                           'Zonkey database file %r is bzip2 compressed;\n'
+                           'uncompressing the archive is recommended:\n'
+                           '  $ bunzip2 "%s"\n' % (filename, filename))
+    except IOError:
+        # Errors are ignored at this stage
+        pass
