@@ -46,9 +46,10 @@ from paleomix.tools.zonkey.common import \
 
 ADMIXTURE_VERSION = versions.Requirement(call=("admixture", "--version"),
                                          search="(\d+)\.(\d+)",
-                                         checks=versions.GE(1, 23))
+                                         checks=versions.GE(1, 3))
 
-PLINK_VERSION = versions.Requirement(call=("plink", "--noweb", "--help"),
+PLINK_VERSION = versions.Requirement(call=("plink", "--noweb", "--help",
+                                           "--out", "/tmp/plink"),
                                      search="v(\d+)\.(\d+)",
                                      checks=versions.GE(1, 7))
 
@@ -64,7 +65,7 @@ TREEMIX_VERSION = versions.Requirement(call=("treemix",),
 class BuildTPEDFilesNode(CommandNode):
     def __init__(self, output_root, table, bamfile, downsample,
                  dependencies=()):
-        cmd = factory.new("build_tped")
+        cmd = factory.new("zonkey_tped")
         cmd.set_option("--name", "Sample")
         cmd.set_option("--downsample", downsample)
         cmd.add_value("%(TEMP_DIR)s")
@@ -157,7 +158,8 @@ class BuildFilteredBEDFilesNode(CommandNode):
                               TEMP_OUT_LOG="indep.log",
                               TEMP_OUT_NOSEX="indep.nosex",
                               TEMP_OUT_PRUNE_IN="indep.prune.in",
-                              TEMP_OUT_PRUNE_OUT="indep.prune.out")
+                              TEMP_OUT_PRUNE_OUT="indep.prune.out",
+                              set_cwd=True)
 
         basename = os.path.basename(output_prefix)
         cmd_filter = AtomicCmd(["plink", "--noweb", "--make-bed",
@@ -171,10 +173,12 @@ class BuildFilteredBEDFilesNode(CommandNode):
                                TEMP_OUT_PREFIX=basename,
                                TEMP_IN_PRUNE="indep.prune.in",
                                TEMP_OUT_NOSEX=basename + ".nosex",
+                               TEMP_OUT_LOG=basename + ".log",
                                OUT_LOG=output_prefix + ".log",
                                OUT_BED=output_prefix + ".bed",
                                OUT_BIM=output_prefix + ".bim",
-                               OUT_FAM=output_prefix + ".fam")
+                               OUT_FAM=output_prefix + ".fam",
+                               set_cwd=True)
 
         CommandNode.__init__(self,
                              description="<BuildFilteredBEDFiles -> '%s.*'>"
@@ -383,7 +387,7 @@ class BuildFreqFilesNode(CommandNode):
         basename = os.path.basename(output_prefix)
 
         plink_cmd = ["plink", "--freq", "--missing", "--noweb",
-                     "--bfile", input_prefix,
+                     "--bfile", os.path.abspath(input_prefix),
                      "--within", "%(TEMP_OUT_CLUST)s",
                      "--out", "%(TEMP_OUT_PREFIX)s"]
 
@@ -398,16 +402,15 @@ class BuildFreqFilesNode(CommandNode):
                           OUT_NOSEX=output_prefix + ".frq.strat.nosex",
                           OUT_LOG=output_prefix + ".frq.strat.log",
                           TEMP_OUT_PREFIX=basename,
-                          CHECK_VERSION=PLINK_VERSION)
+                          CHECK_VERSION=PLINK_VERSION,
+                          set_cwd=True)
 
         gzip = AtomicCmd(["gzip", "%(TEMP_IN_FREQ)s"],
                          TEMP_IN_FREQ=basename + ".frq.strat",
                          OUT_FREQ=output_prefix + ".frq.strat.gz")
 
-        # FIXME! Can be
         self._tfam = tfam
         self._basename = basename
-
         CommandNode.__init__(self,
                              description="<BuildFreqFiles -> '%s.*'"
                              % (output_prefix,),
@@ -647,6 +650,7 @@ numoutevec:        5
 familynames:       YES
 numoutlieriter:    1
 numchrom:          {nchroms}
+numthreads:        1
 """.format(input_prefix=os.path.abspath(self._input_prefix),
            output_prefix=os.path.basename(self._output_prefix),
            nchroms=self._nchroms))
@@ -677,7 +681,7 @@ class PlotPCANode(CommandNode):
                         OUT_PNG=output_prefix + ".png",
                         CHECK_R=RSCRIPT_VERSION,
                         CHECK_R_GGPLOT2=rtools.requirement("ggplot2"),
-                        CHECK_R_LABELS=rtools.requirement("directlabels"),
+                        CHECK_R_LABELS=rtools.requirement("ggrepel"),
                         set_cwd=True)
 
         CommandNode.__init__(self,
@@ -715,14 +719,25 @@ class PlotCoverageNode(CommandNode):
         with open(os.path.join(temp, "contigs.table"), "w") as handle:
             handle.write("ID\tSize\tNs\tHits\n")
 
-            for line in pysam.idxstats(self._input_file):
-                name, size, hits, _ = line.strip().split('\t')
+            # Workaround for pysam < 0.9 returning list, >= 0.9 returning str
+            for line in "".join(pysam.idxstats(self._input_file)).split('\n'):
+                line = line.strip()
+                if not line:
+                    continue
+
+                name, size, hits, _ = line.split('\t')
                 name = contig_name_to_plink_name(name)
                 if name is None or not (name.isdigit() or name == 'X'):
                     continue
+                elif name not in self._contigs:
+                    # Excluding contigs is allowed
+                    continue
 
                 if int(size) != self._contigs[name]['Size']:
-                    raise NodeError("TODO: size mismatch")
+                    raise NodeError("Size mismatch between database and BAM; "
+                                    "expected size %i, found %i for contig %r"
+                                    % (int(size), self._contigs[name]['Size'],
+                                       name))
 
                 row = {
                     'ID': name,
