@@ -76,11 +76,13 @@ class ReportNode(Node):
 
     def _run(self, _config, temp):
         with open(os.path.join(temp, "report.html"), "w") as output_handle:
+            revision = self._data.settings['Revision']
             header = _HTML_HEADER.format(Version=paleomix.__version__,
+                                         Database=revision,
                                          Sidebar=self._build_sidebar())
             output_handle.write(header)
 
-            self._write_overview(output_handle)
+            self._write_intro_and_overview(output_handle)
             self._write_sample_description(output_handle)
 
             if self._has_nuc:
@@ -104,11 +106,17 @@ class ReportNode(Node):
         css_path = paleomix.resources.report("zonkey", "report.css")
         fileutils.copy_file(css_path, os.path.join(self._root, "report.css"))
 
-    def _write_overview(self, output_handle):
-        output_handle.write(_SECTION_HEADER.format(name="overview",
-                                                   title="Overview"))
+    def _write_intro_and_overview(self, output_handle):
+        output_handle.write(_SECTION_HEADER.format(name="intro",
+                                                   title="Introduction"))
+        output_handle.write(_INTRODUCTION)
 
-        overview = _OVERVIEW_HEADER.format(PYSAM=pysam.__version__,
+        output_handle.write(_SECTION_HEADER.format(name="overview",
+                                                   title="Analysis overview"))
+
+        revision = self._data.settings['Revision']
+        overview = _OVERVIEW_HEADER.format(DATABASE=revision,
+                                           PYSAM=pysam.__version__,
                                            SAMTOOLS=_fmt_v(SAMTOOLS_VERSION),
                                            PLINK=_fmt_v(nuclear.PLINK_VERSION),
                                            RSCRIPT=_fmt_v(RSCRIPT_VERSION))
@@ -188,9 +196,10 @@ class ReportNode(Node):
         try:
             groups = self._report.admixture_results(k_groups, incl_ts)
         except admixture.AdmixtureError, error:
-            return "<strong>ERROR: {}</strong".format(error)
+            return _warn("ERROR: {}</strong".format(error))
 
-        if sum((value >= cutoff) for _, value in groups) < 2:
+        n_admixture_candidates = sum((value >= cutoff) for _, value in groups)
+        if n_admixture_candidates < 2:
             return "<strong>No admixture detected.</strong>"
 
         lines = [
@@ -206,7 +215,10 @@ class ReportNode(Node):
 
         lines.append("</ul>")
 
-        if sum((value >= cutoff) for _, value in groups) != 2:
+        if n_admixture_candidates != 2:
+            lines.append(_warn('WARNING: %s-way admixture detected; this may '
+                               'indicate a false-positive result!'
+                               % (n_admixture_candidates,)))
             return "\n            ".join(lines)
 
         percentiles = self._report.admixture_percentiles(data=self._data,
@@ -214,7 +226,7 @@ class ReportNode(Node):
                                                          incl_ts_k=incl_ts)
 
         if not ('Lower' in percentiles or 'Upper' in percentiles):
-            lines.append("WARNING: Could not determine percentiles.")
+            lines.append(_warn("WARNING: Could not determine percentiles."))
 
             return "\n            ".join(lines)
 
@@ -344,15 +356,17 @@ class AnalysisReport(object):
         if self._has_nuc:
             # Summary file generated while building TPED files
             input_files.append(os.path.join(self._root,
+                                            "results",
                                             "plink",
                                             "common.summary"))
 
             for postfix in ('incl_ts', 'excl_ts'):
-                admix_root = os.path.join(self._root, "admixture")
+                admix_root = os.path.join(self._root, "results", "admixture")
 
                 if self._filtered:
                     # Required to count number of SNPs included after filtering
                     input_files.append(os.path.join(self._root,
+                                                    "results",
                                                     "plink",
                                                     postfix + ".bim"))
 
@@ -374,13 +388,16 @@ class AnalysisReport(object):
 
         if self._has_mt:
             input_files.append(os.path.join(self._root,
+                                            "results",
                                             "mitochondria",
                                             "sequences.summary"))
 
         return input_files
 
     def snp_summary(self):
-        summary = read_summary(os.path.join(self._root, "plink",
+        summary = read_summary(os.path.join(self._root,
+                                            "results",
+                                            "plink",
                                             "common.summary"))
 
         if self._filtered:
@@ -397,13 +414,14 @@ class AnalysisReport(object):
 
     def mito_summary(self):
         return read_summary(os.path.join(self._root,
+                                         "results",
                                          "mitochondria",
                                          "sequences.summary"))
 
     def admixture_results(self, k_groups, incl_ts,
                           cutoff=admixture.CUTOFF):
         prefix = "incl_ts" if incl_ts else "excl_ts"
-        filename = os.path.join(self._root, "admixture",
+        filename = os.path.join(self._root, "results", "admixture",
                                 "%s.%i.Q" % (prefix, k_groups))
 
         return admixture.read_admixture_results(filename=filename,
@@ -425,7 +443,9 @@ class AnalysisReport(object):
         sample1, = groups[0]
         sample2, = groups[1]
         delta = abs(max(value for _, value in results) - 0.5)
-        summary = read_summary(os.path.join(self._root, "plink",
+        summary = read_summary(os.path.join(self._root,
+                                            "results",
+                                            "plink",
                                             "common.summary"))
 
         return admixture.get_percentiles(data=data,
@@ -441,6 +461,10 @@ def _fmt_v(requirement):
     return ".".join(map(str, requirement.version))
 
 
+def _warn(text):
+    return "<div class='warning'>%s</div>" % (text,)
+
+
 ###############################################################################
 
 _TS_LABELS = {
@@ -454,7 +478,7 @@ _TS_LABELS = {
 _HTML_HEADER = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml">
 <head>
-<title>PALEOMIX Zonkey v{Version}</title>
+<title>PALEOMIX Zonkey v{Version} - db rev. {Database}</title>
 <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
 <link href="report.css" rel="stylesheet" type="text/css" />
 </head>
@@ -462,7 +486,7 @@ _HTML_HEADER = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
 <a name="top" id="top"></a>
 <center>
   <div id="header">
-    <h1>PALEOMIX Zonkey v{Version}</h1>
+    <h1>PALEOMIX Zonkey v{Version} - db rev. {Database}</h1>
     <h2>A pipeline for detection of F1 hybrids in equids.</h2>
   </div>
   <div id="content">
@@ -483,17 +507,46 @@ _HTML_HEADER = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
 _SECTION_HEADER = """      <h1><a name="{name}" id="{name}"></a>{title}</h1>
 """
 
-_OVERVIEW_HEADER = """
-      <p> Data processed using
-        <a href="https://github.com/pysam-developers/pysam">pysam</a> v{PYSAM},
-        <a href="https://samtools.github.io/">SAMTools</a> v{SAMTOOLS}
-        [<em>Li <em>et al.</em> 2009</em>] and
-        <a href="http://pngu.mgh.harvard.edu/purcell/plink/">PLINK</a> v{PLINK}
-        [<em>Purcell <em>et al.</em> 2007</em>]; plotting was carried out using
-        <a href="https://www.r-project.org/">R</a> v{RSCRIPT}. Additional
-        tools listed below.
+_INTRODUCTION = """
+      <div>
+        <div>
+          The Zonkey Pipeline is a easy-to-use pipeline designed for the
+          analyses of low-coverage, ancient DNA derived from historical
+          equid samples, with the purpose of determining the species of
+          the sample, as well as determining possible hybridization between
+          horses, zebras, and asses. This is accomplished by comparing one
+          or more samples aligned against the <em>Equus caballus</em> 2.0
+          reference sequence with a reference panel of modern equids,
+          including wild and domesticated equids.
+        </div>
+        <br/>
+        <div>
+          For more information, please refer to the
+          <a href="http://paleomix.readthedocs.org/en/latest/zonkey_pipeline/index.html">
+            the documentation for the Zonkey pipeline
+          </a>
+          or
+          <a href="http://paleomix.readthedocs.org/en/latest/">
+            the documentation for the PALEOMIX pipeline,
+          </a>
+          on which the Zonkey pipeline is based.
+        </div>
+        <br/>
+"""
 
-      <div style="display:table;width:100%;">
+_OVERVIEW_HEADER = """
+        <div>
+          Zonkey run using database rev. {DATABASE}. Data processed using
+          <a href="https://github.com/pysam-developers/pysam">pysam</a> v{PYSAM},
+          <a href="https://samtools.github.io/">SAMTools</a> v{SAMTOOLS}
+          [<em>Li <em>et al.</em> 2009</em>] and
+          <a href="http://pngu.mgh.harvard.edu/purcell/plink/">PLINK</a> v{PLINK}
+          [<em>Purcell <em>et al.</em> 2007</em>]; plotting was carried out using
+          <a href="https://www.r-project.org/">R</a> v{RSCRIPT}. Additional
+          tools listed below.
+        </div>
+        <br/>
+        <div style="display:table;width:100%;">
           <div style="display:table-cell;">
 """
 
@@ -552,12 +605,16 @@ _OVERVIEW_MITOCHONDRIA = """
 
 _OVERVIEW_NUCLEAR_COVERAGE = """
           </div>
-          <div style="display:table-cell;width:350px;">
-           <strong>Autosomes vs. sex-chromosomes:</strong>
-           <a href="figures/coverage/coverage.pdf">
-             <img src="figures/coverage/coverage.png"
-                  style="vertical-align:top;horizontal-align:center;">
-           </a>
+          <div style="display:table-cell;width:25%;max-width:350px;">
+           <div>
+             <strong>Autosomes vs. sex-chromosomes:</strong>
+           </div>
+           <div>
+             <a href="figures/coverage/coverage.pdf">
+               <img src="figures/coverage/coverage.png"
+                    style="vertical-align:top;horizontal-align:center;">
+             </a>
+           </div>
 """
 
 _OVERVIEW_FOOTER = """
@@ -790,7 +847,8 @@ _SIDEBAR_HEADER = """
       <h1>Contents</h1>
       <div class="submenu">
         <a href="#">Top</a>
-        <a href="#intro">Overview</a>
+        <a href="#intro">Introduction</a>
+        <a href="#overview">Analysis overview</a>
         <a href="#samples">Reference Panel</a>
 """
 
