@@ -52,10 +52,13 @@ class SummaryNode(Node):
             input_files.update(report.input_files())
             self._reports[sample] = report
 
+        output_prefix = os.path.join(self._root, "summary")
         Node.__init__(self,
+                      description="<SummaryReport -> %r>"
+                      % (output_prefix + '.html',),
                       input_files=input_files,
-                      output_files=(os.path.join(self._root, "summary.html"),
-                                    os.path.join(self._root, "summary.css")),
+                      output_files=(output_prefix + '.html',
+                                    output_prefix + '.css'),
                       dependencies=dependencies)
 
     def _run(self, _config, temp):
@@ -140,7 +143,8 @@ class SummaryNode(Node):
         if "Nuc" in info["Files"]:
 
             for postfix in ('incl_ts', 'excl_ts'):
-                admix_root = os.path.join(self._root, sample, "admixture")
+                admix_root = os.path.join(self._root, sample,
+                                          "results", "admixture")
 
                 for k_groups in (2, 3):
                     filename = os.path.join(admix_root, "%s.%i.Q" % (postfix,
@@ -163,6 +167,7 @@ class SummaryNode(Node):
         output_handle.write(tmpl.format(sample_idx=sample_idx,
                                         n_tests=n_tests,
                                         n_pos_tests=n_pos_tests))
+        output_handle.write(_SAMPLE_OVERVIEW_FOOTER.format(sample=sample))
 
         output_handle.write(_SAMPLE_DATA_HEADER.format(sample_idx=sample_idx))
 
@@ -181,7 +186,9 @@ class SummaryNode(Node):
         lines.append('            <table summary="Admixture overview for sample {}" style="width:125px;">'.format(sample.replace('"', "")))
         lines.append("              <tr>")
         for postfix in ('incl_ts', 'excl_ts'):
-            admix_root = os.path.join(self._root, sample, "admixture")
+            admix_root = os.path.join(self._root, sample,
+                                      "results", "admixture")
+
             for k_groups in (2, 3):
                 filename = os.path.join(admix_root, "%s.%i.Q" % (postfix, k_groups))
 
@@ -287,14 +294,36 @@ _HTML_HEADER = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
         <h1>Contents</h1>
         <div class="submenu">
           <a href="#">Top</a>
+          <a href="#intro">Introduction</a>
           {MenuEntries}
         </div>
       </div>
       <div id="mainbar">
         <h1><a name="introduction" id="introduction"></a>Introduction</h1>
-        <p>
-          TODO
-        </p>
+        <div>
+          <div>
+            The Zonkey Pipeline is a easy-to-use pipeline designed for the
+            analyses of low-coverage, ancient DNA derived from historical
+            equid samples, with the purpose of determining the species of
+            the sample, as well as determining possible hybridization between
+            horses, zebras, and asses. This is accomplished by comparing one
+            or more samples aligned against the <em>Equus caballus</em> 2.0
+            reference sequence with a reference panel of modern equids,
+            including wild and domesticated equids.
+          </div>
+          <br/>
+          <div>
+            For more information, please refer to the
+            <a href="http://paleomix.readthedocs.org/en/latest/zonkey_pipeline/index.html">
+              the documentation for the Zonkey pipeline
+            </a>
+            or
+            <a href="http://paleomix.readthedocs.org/en/latest/">
+              the documentation for the PALEOMIX pipeline,
+            </a>
+            on which the Zonkey pipeline is based.
+          </div>
+        </div>
         <h1><a name="samples" id="samples"></a>Sample Overview</h1>
 """
 
@@ -337,27 +366,32 @@ _SAMPLE_TAB = """
 
 _SAMPLE_OVERVIEW_INCL_NUCL_POSITIVE = """
     <div class="tabContent small" id="sample{sample_idx}.page1">
-      <p>
+      <div>
         Possible admixture detected in {n_pos_tests} of {n_tests} tests.
-      </p>
-    </div>
+      </div>
 """
 
 
 _SAMPLE_OVERVIEW_INCL_NUCL_NEGATIVE = """
     <div class="tabContent small" id="sample{sample_idx}.page1">
-      <p>
+      <div>
         No admixture detected in nuclear genome.
-      </p>
-    </div>
+      </div>
 """
 
 
 _SAMPLE_OVERVIEW_EXCL_NUCL = """
     <div class="tabContent small" id="sample{sample_idx}.page1">
-      <p>
+      <div>
         Nuclear BAM not provided; no admixture were tests perfomed.
-      </p>
+      </div>
+"""
+
+
+_SAMPLE_OVERVIEW_FOOTER = """
+      <div style="text-align:center;">
+          <a href="{sample}/report.html">(Click here for the full report)</a>
+      </div>
     </div>
 """
 
