@@ -315,26 +315,33 @@ def build_pipeline(config, root, nuc_bam, mito_bam, cache):
 
 
 def run_admix_pipeline(config):
-    print_info("Building Zonkey pipeline ", end='')
+    print_info("\nBuilding %i Zonkey pipeline(s):" % (len(config.samples),))
     config.temp_root = os.path.join(config.destination, "temp")
     if not config.dry_run:
         fileutils.make_dirs(config.temp_root)
 
     cache = {}
     nodes = []
-    for sample in config.samples.itervalues():
+    items = config.samples.iteritems()
+    for idx, (name, sample) in enumerate(sorted(items), start=1):
         root = sample["Root"]
         nuc_bam = sample["Files"].get("Nuc")
         mito_bam = sample["Files"].get("Mito")
 
+        genomes = []
+        if mito_bam:
+            genomes.append("MT")
+        if nuc_bam:
+            genomes.append("Nuclear")
+
+        print_info("  %i. %s: %s DNA" % (idx, name, ' and '.join(genomes)))
+
         nodes.extend(build_pipeline(config, root, nuc_bam, mito_bam, cache))
-        print_info(".", end='')
 
     if config.multisample and not config.admixture_only:
         nodes = [summary.SummaryNode(config, nodes)]
-    print_info(".")
 
-    if not run_pipeline(config, nodes, "\nRunning Zonkey ..."):
+    if not run_pipeline(config, nodes, "\nRunning Zonkey:"):
         return 1
 
 
